'use strict';
const fs        = require('fs');
const wait      = require('co-wait');
const moment    = require('moment');
const md5File   = require('md5-file/promise');

function *getS3object( s3Info ) {
  const s3 = new s3Info.AWS.S3();
  const params = {
    Bucket: s3Info.Bucket,
    Key: s3Info.Key
  };
  let s3Object = null;
  try {
    s3Object = yield s3.headObject(params).promise();
  } catch( error ) {}
  return s3Object;
}

function *fileExists( config ) {
  let result = false;
  const s3Object = yield getS3object({
    Bucket  : config.params.s3.bucket,
    Key     : getKeyName( config ),
    AWS     : config.AWS
  });
  result = s3Object != null;
  if( result ) {
    result = s3Object.ContentLength == config.item.stat.size;
  }
  return result;
}

function getKeyName( config ) {
  let key = `${config.params.s3.prefix}/${config.item.path}`;
  return key.replace( /\/\//g, '/' );
}

function upload( config ) {
  const result = new Promise( function( resolve, reject ) {
    const s3 = new config.AWS.S3();
    var readStream = fs.createReadStream( config.item.path );
    const uploadParams = { Bucket : config.params.s3.bucket, Key : getKeyName( config ), Body : readStream };
    const options = { partSize : config.params.s3.partSize, queueSize : config.params.s3.queueSize };
    const upload = s3.upload( uploadParams, options );
    upload.send( function( err, data ) {
      return err ? reject( { error : err } ) : resolve( data );
    });
  });
  return result;
}

module.exports = function *actor( config ) {
  let result  = null;
  let start   = null;
  let seconds = null;
  let kBps    = null;
  let kbps    = null;
  if( ! ( yield fileExists( config ) ) ) {
    start = moment();
    result = yield upload( config );
    seconds = moment().diff( start, 'seconds' );
    kBps = ( config.item.stat.size / 1024 ) / seconds;
    kbps = Math.round( kBps * 8 * 100 ) / 100;
  } else {
    result = 'exists';
  }

  return { item : config.item, result, seconds, kbps };
};

