'use strict';
const chanel  = require('chanel');
const flow    = require('co-flow');
const co      = require('co');
const AWS     = require('aws-sdk');
const tunnel   = require('tunnel');

const fs      = require('./fs');
const s3Pull  = require('./s3Pull');
const actor   = require('./actor');
const pull    = require('./pull');

let ch = null;

function log( ob ) {
  console.log( JSON.stringify( ob ) );
}

function *gotResult( chLocal, queue, params ) {
  let result = null;
  try {
    result = yield chLocal;
  } catch( error ) {
    result = error;
    ch.push( runActor( { item : error.item, params, AWS } ) );
  }
  log( { result } );
}

function runActor( config ) {
  return function(done) {
    co( config.params.direction == 'push' ? actor( config ) : pull( config ) )
    .then( function( value ) {
      done( null, value );
    })
    .catch( function( error ) {
      done( null, error );
    });
  };
}

module.exports = function*( params ) {
  ch = chanel( params.chanel );
  AWS.config.update({
    credentials : params.credentials,
    region      : params.region,
    httpOptions : { agent : tunnel.httpsOverHttp({ proxy : params.proxy }) }
  });
  let queue = params.direction == 'push' ? fs( params ) : s3Pull( params, AWS );
  let next = null;
  while( true ) {
    if( ! ch.pushable ) {
      log( { message : 'Waiting for pool to have more room...' } );
      yield gotResult( ch, queue, params, AWS );
    }
    next = queue.shift();
    if( next ) {
      log( { message : 'New Item', item : next } );
      ch.push( runActor( { item : next, params, AWS } ) );
    } else {
      if( next == undefined && ch.readable ) {
        log( { message : 'Waiting for pool to empty...' } );
        yield gotResult( ch, queue, params, AWS );
      } else {
        log( { message : 'Waiting for next item...' } );
        yield flow.wait(1000);
      }
    }
  }
};

if (!('toJSON' in Error.prototype))
Object.defineProperty(Error.prototype, 'toJSON', {
  value: function () {
    var alt = {};
    Object.getOwnPropertyNames(this).forEach(function (key) {
        alt[key] = this[key];
    }, this);

    return alt;
  },
  configurable: true,
  writable: true
});