'use strict';
const fs        = require('fs');
const _         = require('lodash');
const wait      = require('co-wait');
const moment    = require('moment');
const md5File   = require('md5-file/promise');

function *getS3object( s3Info ) {
  const s3 = new s3Info.AWS.S3();
  const params = {
    Bucket: s3Info.Bucket,
    Key: s3Info.Key
  };
  let s3Object = null;
  try {
    s3Object = yield s3.getObject(params).promise();
  } catch( error ) {}
  return s3Object;
}

function *fileInfo( config ) {
  let result  = {};
  let stats   = null;
  const item  = config.item;
  const re    = new RegExp( `^${config.params.s3.prefix}\/(.*)$` );
  let path    = item.Key.replace( re, function( match, $1, offset, original ) {
    return $1;
  });
  for( let downloadPrefix of _.keys( config.params.s3.download ) ) {
    if( path.startsWith( downloadPrefix ) ) {
      path = path.replace( downloadPrefix, config.params.s3.download[ downloadPrefix ] );
      break;
    }
  }
  result.path = path.replace( /\/\//g, '/' );
  try {
    stats = fs.statSync( path );
  } catch( error ) { }
  result.exists = stats != null;
  if( result.exists ) {
    result.exists = stats.size == config.item.Size;
  }
  return result;
}

function getKeyName( config ) {
  let key = `${config.params.s3.prefix}/${config.item.path}`;
  return key.replace( /\/\//g, '/' );
}

function download( config, path ) {
  return new Promise( function( resolve, reject ) {
    const s3 = new config.AWS.S3();
    const writeStream = fs.createWriteStream( path );
    const params = { Bucket : config.params.s3.bucket, Key : config.item.Key };
    const readStream = s3.getObject( params ).createReadStream();

    readStream.on('end', function () {
      resolve( { path, item : config.item } );
    });

    writeStream.on('error', function (err) {
      reject( err );
    });

    readStream.pipe( writeStream );
  });
}

module.exports = function *actor( config ) {
  let result  = null;
  let start   = null;
  let seconds = null;
  let kBps    = null;
  let kbps    = null;
  const file = yield fileInfo( config );
  if( ! file.exists ) {
    start = moment();
    result = yield download( config, file.path );
    seconds = moment().diff( start, 'seconds' );
    kBps = ( config.item.Size / 1024 ) / seconds;
    kbps = Math.round( kBps * 8 * 100 ) / 100;
  } else {
    result = 'exists';
  }

  return { item : config.item, result, seconds, kbps };
};

