'use strict';
const co = require('co');

const queue = [];

function *getObjectList( params, AWS, ContinuationToken ) {
  const s3 = new AWS.S3();
  const callParams = {
    Bucket : params.s3.bucket,
    ContinuationToken,
    Prefix: params.s3.prefix
  };
  const result = yield s3.listObjectsV2( callParams ).promise();
  for( let item of result.Contents ) {
    queue.push( item );
  }
  return result.NextContinuationToken;
}

function *getInitial( params, AWS ) {
  let ContinuationToken = null;
  while( ! ContinuationToken ) {
    ContinuationToken = yield getObjectList( params, AWS, ContinuationToken );
  }
}

function *main(params, AWS)  {
  return yield getInitial( params, AWS );
}

function handleSuccess( value ) { return value; }
function handleError( error ) { throw error; }

module.exports = function( params, AWS ) {
  co( main( params, AWS ) ).then( handleSuccess ).catch( handleError );
  return queue;
};
